/*	Public domain	*/

struct sk_tool_keybinding;
struct sk_tool_mousebinding;
struct sk_view;
struct ag_static_icon;

typedef struct sk_tool_ops {
	const char *name;
	const char *desc;
	struct ag_static_icon *icon;
	size_t len;
	Uint flags;
#define SK_MOUSEMOTION_NOSNAP	0x01	/* Ignore snapping in mousemotion */
#define SK_BUTTONUP_NOSNAP	0x02	/* Ignore snapping in buttonup */
#define SK_BUTTONDOWN_NOSNAP	0x04	/* Ignore snapping in buttondown */
#define SK_BUTTON_NOSNAP	(SK_BUTTONUP_NOSNAP|SK_BUTTONDOWN_NOSNAP)
#define SK_NOSNAP		(SK_BUTTON_NOSNAP|SK_MOUSEMOTION_NOSNAP)

	void (*init)(void *);
	void (*destroy)(void *);
	void (*edit)(void *, void *);
	int (*mousemotion)(void *, M_Vector3 pos, M_Vector3 vel, int btn);
	int (*mousebuttondown)(void *, M_Vector3 pos, int btn);
	int (*mousebuttonup)(void *, M_Vector3 pos, int btn);
	int (*keydown)(void *, int ksym, int kmod);
	int (*keyup)(void *, int ksym, int kmod);
} SK_ToolOps;

typedef struct sk_tool {
	const SK_ToolOps *ops;
	struct sk_view *skv;			/* Associated view */
	void *p;				/* User-supplied pointer */
	AG_Window *win;				/* Edition window (if any) */
	AG_Button *trigger;			/* Trigger button (XXX) */
	AG_SLIST_HEAD_(sk_tool_keybinding) kbindings;
	AG_SLIST_HEAD_(sk_tool_mousebinding) mbindings;
	AG_TAILQ_ENTRY(sk_tool) tools;
} SK_Tool;

typedef struct sk_tool_keybinding {
	AG_KeyMod mod;
	AG_KeySym key;
	int edit;
	int (*func)(SK_Tool *, AG_KeySym k, int s, void *);
	void *arg;
	AG_SLIST_ENTRY(sk_tool_keybinding) kbindings;
} SK_ToolKeyBinding;

typedef struct sk_tool_mousebinding {
	int button;
	int edit;
	int (*func)(SK_Tool *, int b, int s, M_Vector3 pos, void *);
	void *arg;
	AG_SLIST_ENTRY(sk_tool_mousebinding) mbindings;
} SK_ToolMouseBinding;

#define SKTOOL(t) ((SK_Tool *)(t))
#define SK_CURTOOL(skv) (skv)->curtool != NULL ? (skv)->curtool : (skv)->deftool != NULL ? (skv)->deftool : NULL

/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC void SK_ToolInit(SK_Tool *);
extern DECLSPEC void SK_ToolDestroy(SK_Tool *);
extern DECLSPEC AG_Window *SK_ToolWindow(void *, const char *);
extern DECLSPEC void SK_ToolBindKey(void *, AG_KeyMod, AG_KeySym, int (*)(SK_Tool *, AG_KeySym, int, void *), void *);
extern DECLSPEC void SK_ToolBindMouseButton(void *, int, int (*)(SK_Tool *, int, int, M_Vector3, void *), void *);
extern DECLSPEC void SK_ToolUnbindKey(void *, AG_KeyMod, AG_KeySym);
__END_DECLS
/* Close generated block */
